/* kpm.h
 *
 * Copyright (C) 2023 OpenKylin, Inc
 */

#ifndef __KPM_H__
#define __KPM_H__

#include <list>
#include <memory>
#include <string>
#include <functional>
#include "common/kmerror.h"

namespace KPM
{
    typedef struct _PackageInfo
    {
        std::string name;
        std::string branch;
        std::string version;
        size_t packageSize;
    } packageInfo;

    typedef struct _ProgressStatus
    {
        std::string refId;
        int progress;
        std::string speed;
        int timeout;
    } progressStatus;

    typedef std::function <void (std::string progress_status)> progressCallBack;


    class KaimingPackageManager
    {
    public:
        KaimingPackageManager();
        ~KaimingPackageManager();

        /// @brief 应用/运行时 是否安装
        /// @param name 要查询的名称 （top.openkylin.Music）
        /// @return
        bool isInstalled(const std::string name);

        /// @brief 安装应用
        /// @param name 要安装的应用名 （top.openkylin.Music）
        /// @param branch 要安装的应用分支 （填 "" 为自动获取）
        /// @param progress_callback 安装进度回调
        /// @param err_no 错误号
        /// @return 是否安装成功
        bool install(const std::string name, const std::string branch, progressCallBack progress_callback, KMErrorCodeEnum &err_no);

        /// @brief 卸载应用
        /// @param name 应用名
        /// @param branch 要卸载的分支 （当前默认设置 "" 就可，删除所有分支）
        /// @param err_no
        /// @return 是否卸载成功
        bool uninstall(const std::string name, const std::string branch, KMErrorCodeEnum &err_no);

        /// @brief 应用是否可更新
        /// @param name 应用名
        /// @return
        bool isUpgradable(const std::string name);

        /// @brief 应用更新
        /// @param name 要安装的应用名 （top.openkylin.Music）
        /// @param branch 要更新的应用分支 （填 "" 为自动获取）
        /// @param progress_callback 更新进度回调
        /// @param err_no
        /// @return 是否更新成功
        bool upgrade(const std::string name, const std::string branch, progressCallBack progress_callback, KMErrorCodeEnum &err_no);

        /// @brief 获取远程仓库包信息
        /// @param name 要获取的包信息
        /// @return 获取到的包信息列表
        std::list<packageInfo> getReomteInfo(const std::string name);

        /// @brief 获取本地安装包信息
        /// @param name 要获取的包信息
        /// @return 获取到的包信息列表
        std::list<packageInfo> getLocalInfo(const std::string name);

    private:
        class Private;
        std::shared_ptr<Private> d_ptr;
    };

}

#endif // !__KPM_H__
